@@ -22,9 +22,11 @@
 import greentest
 import unittest
 import time
+import re
 import gevent
 from gevent import core
 from gevent import socket
+from gevent.hub import Waiter
 import signal
 
 DELAY = 0.1
@@ -95,6 +97,8 @@ class TestShutdown(unittest.TestCase):
         assert 'hub' not in gevent.hub._threadlocal.__dict__
 
     def test(self):
+        # make sure Hub is started. For the test case when hub is not started, see test_hub_shutdown.py
+
         assert not gevent.hub.get_hub().dead
         self._shutdown()
         self.assert_no_hub()
@@ -142,6 +146,37 @@ class TestSignal(greentest.TestCase):
             assert str(ex) == 'TestSignal', ex
 
 
+class TestWaiter(greentest.GenericWaitTestCase):
+
+    def setUp(self):
+        super(TestWaiter, self).setUp()
+        self.waiter = Waiter()
+
+    def wait(self, timeout):
+        evt = core.timer(timeout, self.waiter.switch, None)
+        try:
+            return self.waiter.get()
+        finally:
+            evt.cancel()
+
+    def test(self):
+        waiter = self.waiter
+        self.assertEqual(str(waiter), '<Waiter greenlet=None>')
+        waiter.switch(25)
+        self.assertEqual(str(waiter), '<Waiter greenlet=None value=25>')
+        self.assertEqual(waiter.get(), 25)
+
+        waiter = Waiter()
+        waiter.throw(ZeroDivisionError)
+        assert re.match('^<Waiter greenlet=None exc_info=.*ZeroDivisionError.*$', str(waiter)), str(waiter)
+        self.assertRaises(ZeroDivisionError, waiter.get)
+
+        waiter = Waiter()
+        gevent.spawn(waiter.get)
+        gevent.sleep(0)
+        assert str(waiter).startswith('<Waiter greenlet=<Greenlet at '), str(waiter)
+
+
 if __name__=='__main__':
     greentest.main()
 
